from typing import List, Optional

class OperationNameResult:
    optional_int: Optional[int]
    list_of_int: list[int]
    list_of_optional_int: list[Optional[int]]
    optional_list_of_optional_int: Optional[list[Optional[int]]]
