query OperationName {
  union {
    ... on Animal {
      age
    }
    ... on Person {
      name
    }
  }
  optionalUnion {
    ... on Animal {
      age
    }
    ... on Person {
      name
    }
  }
}
